// CPU
#if (defined(KUSE64) && !defined(KUSE32))
	typedef long long KInt;
	typedef unsigned long long KUInt;
#elif (defined(KUSE32) && !defined(KUSE64))
	typedef int KInt;
	typedef unsigned int KUInt;
#else
	typedef void KInt;	// gG[o悤.
	typedef void KUInt;
#endif

// OS
#if (!defined(KWINDOWS) && !defined(KPOSIX))
	#if (defined(__linux__) || defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(__DragonFly__))
		#define KPOSIX	1
	#elif (defined(WIN32))
		#define KWINDOWS	1
	#endif
#endif

// graphics
#if (!defined(KGDI32) && !defined(KSDL1) && !defined(KSDL2) && !defined(KCONSONLY))
	#if (defined(KWINDOWS))
		#define KGDI32	1
	#elif (defined(KPOSIX))
		#define KSDL2	1
	#else
		#define KCONSONLY	1
	#endif
#endif

// ABI
#if (defined(KUSE64) && !defined(KABI_MS64) && !defined(KABI_SYSV64))
	#if (defined(KWINDOWS))
		#define KABI_MS64	1
	#elif (defined(KPOSIX))
		#define KABI_SYSV64	1
	#endif
#endif

#define STATIC	static

//////////////////////////////////////////////////////////////////////////////// KPRINTF_FORMAT

#if (defined(KUSE32))
	#define KPRINTF_FORMAT(x, y, z)		x
#elif (defined(KUSE64) && defined(KWINDOWS))
	#define KPRINTF_FORMAT(x, y, z)		y
#elif (defined(KUSE64) && defined(KPOSIX))
	#define KPRINTF_FORMAT(x, y, z)		z
#endif

//////////////////////////////////////////////////////////////////////////////// kerrorExit

STATIC void kerrorExit(const char *f, ...)
{
    va_list ap;
    va_start(ap, f);
    vfprintf(stderr, f, ap);
    fprintf(stderr, "\n");
    va_end(ap);
    exit(EXIT_FAILURE);
}

STATIC void *kerrorExitP0(void *p, const char *f, ...)
{
	if (p != 0)
		return p;
    va_list ap;
    va_start(ap, f);
    vfprintf(stderr, f, ap);
    fprintf(stderr, "\n");
    va_end(ap);
    exit(EXIT_FAILURE);
}

//////////////////////////////////////////////////////////////////////////////// KMemPool

// rbgɂȂA8oCgPʂł.

#define KMEMPOOL_AFLOG	0

typedef struct KMemPool_ {
	void *p[27 * 4];
	KInt c[27 * 4], siz1;
	void *child, *next, *parent, *src[2], *dest;
} KMemPool;

STATIC KInt KMemPool_table[27 * 4];

STATIC void KMemPool_initTable()
{
	int i, j;
	for (i = 0; i < 8; i++)
		KMemPool_table[i] = i * 8 + 8;
	for (; i < 27 * 4; i += 4) {
		KInt b = KMemPool_table[i - 1], b2 = b >> 2;
		for (j = 0; j < 4; j++) {
			b += b2;
			KMemPool_table[i + j] = b;
		}
	}
}

STATIC int KMemPool_index(KInt s)
{
    KInt a, i;
    if (s < 0x20) {
        if (s > 0)
            return (s - 1) >> 3; // 0,1,2,3
        return 0;
    }
	#if (defined(KUSE64))
	    if (s > 0x80000000LL)
    	    kerrorExit("KMemPool: too large size: s="  KPRINTF_FORMAT("%d", "%I64d", "%ld"), s);
	#endif
#if 0	// ʂ̏.
    a = kgetmsb32(s) - 1;
    i = kpopcnt32(a) * 4 - 13;
    s += a >> 2;
	a++;
    if ((s & (a >> 1)) != 0) i += 2;
    if ((s & (a >> 2)) != 0) i++;
#else	// CCWJč.
    // a = kgetmsb64(s) - 1;
	a = s >> 1;
	a |= a >> 1;
	a |= a >> 2;
	a |= a >> 4;
	a |= a >> 8;
	a |= a >> 16;
	#if (defined(KUSE64))
		a |= a >> 32;
	#endif
    // i = kpopcnt64(a);
	#if (defined(KUSE64))
	    i = a - (a >> 1 & 0x5555555555555555ULL);
    	i = (i & 0x3333333333333333ULL) + (i >> 2 & 0x3333333333333333ULL);
    	i = (i + (i >> 4)) & 0x0f0f0f0f0f0f0f0fULL;
    	i = (i * 0x0101010101010101ULL) >> 56;
	#elif (defined(KUSE32))
	    i = a - (a >> 1 & 0x55555555U);
    	i = (i & 0x33333333U) + (i >> 2 & 0x33333333U);
    	i = (i + (i >> 4)) & 0x0f0f0f0fU;
    	i = (i * 0x01010101U) >> 24;
	#endif
    s += a >> 2;
    s >>= i - 2;
    i = i * 4 + s - 21;
#endif
    return i;
}

STATIC void *KMemPool_alloc(KMemPool *w, KInt siz);

STATIC void *KMemPool_alcsub0(KMemPool *w, KInt siz)
{
	KInt *p;
	if (w->parent != 0) {
		p = KMemPool_alloc(w->parent, siz);
		p[1] = siz;
		p[3] = ((KInt) p) + 32;
	} else {
		p = kerrorExitP0(malloc(siz + 32), "KMemPool: out of memory: siz=" KPRINTF_FORMAT("%d", "%I64d", "%ld"), siz);
		p[1] = siz + 32;
		p[3] = ((KInt) p) + 32;
		if ((p[3] & 31) > 0)
			p[3] += 32 - (p[3] & 31);
	}
	return p;
}

STATIC void *KMemPool_alcsub1(KMemPool *w, KInt siz)
{
	if (siz >= w->siz1) {
		char *p = KMemPool_alcsub0(w->parent, siz + 32);
		*((void **) p) = w->src[1];
		w->src[1] = (void *) p;
		return p + 32;
	}
	KInt *p = w->src[0], *p0 = (KInt *) &w->src[0];
	while (p != 0) {
		if (siz <= p[2]) {
			void *q = (void *) p[3];
			p[3] += siz;
			p[2] -= siz;
			if (siz == p[2]) {
				*p0 = p[0];
				*p = (KInt) w->src[1];
				w->src[1] = (void *) p;
			}
			return q;
		}
		p0 = p;
		p = (KInt *) p[0];
	}
	p = KMemPool_alcsub0(w, w->siz1 + 32);
	*p = (KInt) w->src[0];
	w->src[0] = (void *) p;
	p[2] = w->siz1 - siz;
	p[3] += siz;
	return p + 32 / sizeof (KInt);
}

STATIC void *KMemPool_allocIdx(KMemPool *w, int i)
{
	int j;
	KInt s1 = KMemPool_table[i];
	char *p = w->p[i];
	if (p != 0)
		w->p[i] = *((void **) p);
	else {
		w->c[i]++;
		if ((s1 & 31) == 0) {
			p = KMemPool_alcsub1(w, s1);
		} else {
			// 0, 1, 2, 4, 5, 6, 8, 10
			if (i == 0) {	// 8x4=32.
				p = KMemPool_alcsub1(w, 32);
				*((void **) (p + 24)) = 0;
				*((void **) (p + 16)) = p + 24;
				*((void **) (p +  8)) = p + 16;
				w->p[i] = p + 8;
				w->c[i] += 3;
			} else {
				KInt s2 = 32 << (i >> 2);
				p = KMemPool_alcsub1(w, s2);
				j = KMemPool_index(s2 - s1);
				*((void **) (p + s1)) = w->p[j];
				w->p[j] = p + s1;
				w->c[j]++;
			}
		}
	}
	#if (KMEMPOOL_AFLOG != 0)
		printf("alc(#%d, %08x) ", i, (int) (KInt) p);
	#endif
	return p;
}

STATIC void *KMemPool_alloc(KMemPool *w, KInt siz)
{
	return KMemPool_allocIdx(w, KMemPool_index(siz));
}

STATIC void *KMemPool_allocP0(KMemPool *w, KInt siz, void *p)
{
	if (p == 0)
		p = KMemPool_allocIdx(w, KMemPool_index(siz));
	return p;
}

STATIC KMemPool *KMemPool_freeIdx(KMemPool *w, int i, void *p)
{
	#if (KMEMPOOL_AFLOG != 0)
		printf("fre(#%d, %08x) ", i, (int) (KInt) p);
	#endif
	*((void **) p) = w->p[i];
	w->p[i] = p;
	return w;
}

STATIC KMemPool *KMemPool_free(KMemPool *w, KInt siz, void *p)
{
	int i = KMemPool_index(siz);
	#if (KMEMPOOL_AFLOG != 0)
		printf("fre(#%d, %08x) ", i, (int) (KInt) p);
	#endif
	*((void **) p) = w->p[i];
	w->p[i] = p;
	return w;
}

STATIC void *KMemPool_realloc(KMemPool *w, KInt s0, KInt s1, void *p)
{
	if (p == 0)
		return KMemPool_alloc(w, s0);
	int i0 = KMemPool_index(s0);
	int i1 = KMemPool_index(s1);
	if (i0 != i1) {
		void *p0 = p;
		p = KMemPool_alloc(w, s1);
		if (s0 > s1)
			s0 = s1;
		if (s0 > 0)
			memcpy(p, p0, s0);
		#if (KMEMPOOL_AFLOG != 0)
			printf("fre(#%d, %08x):%08x ", i0, (int) (KInt) p0, (int) (KInt) w->p[i0]);
		#endif
		*((void **) p0) = w->p[i0];
		w->p[i0] = p0;
	}
	return p;
}

STATIC void *KMemPool_allocC(KMemPool *w, KInt siz)
{
	void *p = KMemPool_alloc(w, siz);
	if (siz > 0)
		memset(p, 0, siz);
	return p;
}

STATIC void *KMemPool_reallocC(KMemPool *w, KInt s0, KInt s1, void *p)
{
	if (p == 0)
		s0 = 0;
	p = KMemPool_realloc(w, s0, s1, p);
	if (s0 < s1)
		memset(((char *) p) + s0, 0, s1 - s0);
	return p;
}

STATIC KMemPool *KMemPool_addDest(KMemPool *w, void *f, void *p0, void *p1)
{
	void **p = KMemPool_allocIdx(w, 3);	// 32bytes.
	p[0] = w->dest;
	w->dest = p;
	p[1] = f;
	p[2] = p0;
	p[3] = p1;
	return w;
}

STATIC KMemPool *KMemPool_delDest(KMemPool *w, void *f, void *p0, void *p1)
{
	void **q0 = &w->dest, **q = w->dest;
	while (q != 0) {
		if (q[1] == f && q[2] == p0 && q[3] == p1) {
			*q0 = (void **) *q;
			KMemPool_freeIdx(w, 3, q);
			return w;
		}
		q0 = q;
		q = (void **) *q;
	}
	return 0;
}

STATIC KMemPool *KMemPool_init(KMemPool *w, KMemPool *p)
{
	w = KMemPool_allocP0(p, sizeof (KMemPool), w);
	memset(w, 0, sizeof (KMemPool));
	w->siz1 = 4 * 1024 * 1024;
	w->parent = p;
	if (p == 0)
		KMemPool_initTable();
	else {
		w->next = p->child;
		p->child = w;
	}
	return w;
}

STATIC void *KMemPool_dest(KMemPool *w, KInt force)
{
	int i, j;
	KInt *p = w->dest;
	while (p != 0) {
		void *q = p;
		((void (*)(KInt, KInt)) p[1])(p[2], p[3]);
		p = (KInt *) *p;
		KMemPool_freeIdx(w, 3, q);
	}
	if (force == 0) {
		for (i = 0; i < 27 * 4; i++) {
			if (w->c[i] == 0) continue;
			p = w->p[i];
			j = 0;
			while (p != 0) {
				p = (KInt *) *p;
				j++;
			}
			if (w->c[i] != j) {
				kerrorExit("KMemPool_dest: memory leak error: w=" KPRINTF_FORMAT("%08x", "%016I64x", "%016lx") ", i=%d, siz1=%d, c=%d, n=%d",
					(KInt) w, i, KMemPool_table[i], w->c[i], j);
			}
			w->c[i] = 0;
			w->p[i] = 0;
		}
	}
	for (i = 0; i < 2; i++) {
		p = w->src[i];
		while (p != 0) {
			KInt *q = (KInt *) p[0];
			if (w->parent != 0)
				KMemPool_free(w->parent, p[1], p);
			else
				free(p);
			p = q;
		}
		w->src[i] = 0;
	}
	return w;
}

STATIC void KMemPool_destFr(KMemPool *w, KInt force)
{
	KMemPool_free(w->parent, sizeof (KMemPool), KMemPool_dest(w, force));
}

STATIC KMemPool *KMemPool_initAd(KMemPool *w, KMemPool *p, KInt force)
{
	w = KMemPool_init(w, p);
	KMemPool_addDest(p, KMemPool_dest, w, (void *) force);
	return w;
}

STATIC KMemPool *KMemPool_initAr(KMemPool *p, KInt force)
{
	KMemPool *w = KMemPool_init(0, p);
	KMemPool_addDest(p, KMemPool_destFr, w, (void *) force);
	return w;
}

#define KMEMPOOL_SAFESMALL 1

STATIC KInt kminInt(KInt i, KInt j);

STATIC KMemPool *KMemPool_dump(KMemPool *w, FILE *fp)
{
	int i, j;
	KInt *p;
	#if (defined(KUSE64) && defined(KWINDOWS))
		fprintf(fp, "KMemPool_dump: %016I64x\n", (KInt) w);
		fprintf(fp, "  parent=%08x%08x\n", (int) (((KInt) w->parent) >> 32), (int) (KInt) w->parent);
	#endif
	for (j = 0; j < 2; j++) {
		fprintf(fp, "  src%d:\n", j);
		p = w->src[j];
		i = 0;
		while (p != 0) {
			#if (defined(KUSE64))
				fprintf(fp, "    #%04d: adr0=%08x%08x siz1=%08x", i, (int) (((KInt) p) >> 32), (int) (KInt) p, (int) p[1]);
				if (i == 0)
					fprintf(fp, " siz=%08x adr=%08x%08x", (int) p[2], (int) (p[3] >> 32), (int) p[3]);
			#endif
			fprintf(fp, "\n");
			i++;
			p = (KInt *) p[0];
		}
	}
	fprintf(fp, "  pool:\n");
	for (i = 0; i < 27 * 4; i++) {
		if (w->c[i] == 0) continue;
		int cc = w->c[i] + 1;
		p = w->p[i];
		j = 0;
		KInt *pp = w->src[0];
		while (p != 0) {
			#if (KMEMPOOL_SAFESMALL != 0)
				if ((char *) p < (char *) pp || pp[1] + (char *) pp <= (char *) p) break;
			#endif
			p = (KInt *) *p;
			j++;
			if (cc < j) break;
		}
		#if (defined(KUSE64))
			fprintf(fp, "    #%03d: siz1=%08x c=%08x n=%08x :", i, (int) KMemPool_table[i], (int) w->c[i], j);
			p = w->p[i];
			j = kminInt(j ,8);
			for (cc = 0; cc <= j; cc++) {
				fprintf(fp, " %08x", (int) (KInt) p);
				#if (KMEMPOOL_SAFESMALL != 0)
					if ((char *) p < (char *) pp || pp[1] + (char *) pp <= (char *) p) break;
				#endif
				p = (KInt *) *p;
			}
		#endif
		fprintf(fp, "\n");
	}
	fprintf(fp, "  dest:\n");
	p = w->dest;
	i = 0;
	while (p != 0) {
		#if (defined(KUSE64) && defined(KWINDOWS))
			fprintf(fp, "    #%04d: adr0=%016I64x f=%016I64x p0=%016I64x p1=%016I64x\n", i, (KInt) p, (KInt) p[1], (KInt) p[2], (KInt) p[3]);
		#endif
		i++;
		p = (KInt *) p[0];
	}
	fprintf(fp, "\n");
	return w;
	// qsrc0/1̏Wve΁AVXeŜŏdȂ邱Ƃł.
}

//////////////////////////////////////////////////////////////////////////////// kqsort

STATIC void *kqsort_optStatic;
STATIC int (*kqsort_cmpStatic)(const void *, const void *, void *);

STATIC int kqsort_cmp(const void *a, const void *b)
{
	return kqsort_cmpStatic(a, b, kqsort_optStatic);
}

STATIC void kqsort(void *base, int num, int size, void *cmp, void *opt)
{
    kqsort_optStatic = opt;
    kqsort_cmpStatic = (int (*)(const void *, const void *, void *)) cmp;
    qsort(base, num, size, kqsort_cmp);
}

//////////////////////////////////////////////////////////////////////////////// misc

STATIC int kpopCount64(unsigned long long b)
{
    b -= b >> 1 & 0x5555555555555555ULL;
    b = (b & 0x3333333333333333ULL) + (b >> 2 & 0x3333333333333333ULL);
    b = (b + (b >> 4)) & 0x0f0f0f0f0f0f0f0fULL;
    return (b * 0x0101010101010101ULL) >> 56;
}

STATIC int kpopCount32(unsigned int b)
{
    b -= b >> 1 & 0x55555555U;
    b = (b & 0x33333333U) + (b >> 2 & 0x33333333U);
    b = (b + (b >> 4)) & 0x0f0f0f0fU;
    return (b * 0x01010101U) >> 24;
}

STATIC unsigned long long kgetMsb64(unsigned long long i)
{
    i >>= 1;
    i |= i >> 1;
    i |= i >> 2;
    i |= i >> 4;
    i |= i >> 8;
    i |= i >> 16;
	i |= i >> 32;
    return i + 1;
}

STATIC unsigned int kgetMsb32(unsigned int i)
{
    i >>= 1;
    i |= i >> 1;
    i |= i >> 2;
    i |= i >> 4;
    i |= i >> 8;
    i |= i >> 16;
    return i + 1;
}

STATIC unsigned int kfnv32(const void *p, KInt l)
{
	const unsigned char *q = (const unsigned char *) p;
    unsigned int h = 0x811c9dc5;
	KInt i;
    for (i = 0; i < l; i++)
		h = (h * 0x01000193) ^ q[i];
    return h;
}

STATIC unsigned long long kfnv64(const void *p, KInt l)
{
	const unsigned char *q = (const unsigned char *) p;
    unsigned long long h = 0xcbf29ce484222325ULL;
	KInt i;
    for (i = 0; i < l; i++)
		h = (h * 0x00000100000001b3ULL) ^ q[i];
    return h;
}

STATIC KInt kminInt(KInt i, KInt j)
{
	if (i <= j) return i;
	return j;
}

STATIC KInt kmaxInt(KInt i, KInt j)
{
	if (i >= j) return i;
	return j;
}

STATIC KInt kstrlenCutSpcLf(KInt l, const char *p)
{
	while (l > 0) {
		if (p[l - 1] != ' ' && p[l - 1] != '\t' && p[l - 1] != '\n' && p[l - 1] != '\r')
			break;
		l--;
	}
	return l;
}

STATIC KInt kstrtoUInt(KInt l, const char *n, char **e, int b)
{
	char s[4032], *ee;
	if (l > (KInt) (sizeof s) - 1)
		l = (sizeof s) - 1;
	memcpy(s, n, l);
	s[l] = '\0';
	#if (defined(KUSE64))
		l = strtoull(s, &ee, b);
	#elif (defined(KUSE32))
		l = strtoul(s, &ee, b);
	#endif
	if (e != 0)
		*e = ee - s + (char *) n;
	return l;
}

STATIC KInt kstrtoInt(KInt l, const char *n, char **e, int b)
{
	char s[4032], *ee;
	if (l > (KInt) (sizeof s) - 1)
		l = (sizeof s) - 1;
	memcpy(s, n, l);
	s[l] = '\0';
	#if (defined(KUSE64))
		l = strtoull(s, &ee, b);
	#elif (defined(KUSE32))
		l = strtol(s, &ee, b);
	#endif
	if (e != 0)
		*e = ee - s + (char *) n;
	return l;
}

STATIC void ktestUseBit()
{
	if (sizeof (int *) != sizeof (KInt))
		kerrorExit("ktestUseBit: build mode error");
}

#if (defined(KUSE64))
	#define kpopCountU		kpopCount64
	#define kgetMsbU		kgetMsb64
	#define kfnvU			kfnv64
#elif (defined(KUSE32))
	#define kpopCountU		kpopCount32
	#define kgetMsbU		kgetMsb32
	#define kfnvU			kfnv32
#endif

static unsigned int kxorshift32_i = 2463534242;

STATIC void kxorshift32_seed(unsigned int s)
{
	kxorshift32_i = 2463534242 ^ s;
}

STATIC unsigned int kxorshift32()
{
	unsigned int i = kxorshift32_i;
	i = i ^ (i << 13);
	i = i ^ (i >> 17);
	i = i ^ (i <<  5);
	kxorshift32_i = i;
	return i;
}

static unsigned long long kxorshift64_i = 88172645463325252ULL;

STATIC void kxorshift64_seed(unsigned int s)
{
	kxorshift64_i = 88172645463325252ULL ^ s;
}

STATIC unsigned long long kxorshift64()
{
	unsigned int i = kxorshift64_i;
	i = i ^ (i << 7);
	i = i ^ (i >> 9);
	kxorshift64_i = i;
	return i;
}

//////////////////////////////////////////////////////////////////////////////// KVarArea

typedef struct KVarArea_ {
	KInt s, s1;
	char *p;
	KMemPool *mp;
} KVarArea;

STATIC KVarArea *KVarArea_init(KVarArea *w, KMemPool *mp, KInt s0)
{
	int i = KMemPool_index(s0);
	w = KMemPool_allocP0(mp, sizeof (KVarArea), w);
	w->s1 = KMemPool_table[i];
	w->s = s0;
	w->p = KMemPool_allocIdx(mp, i);
	w->mp = mp;
	return w;
}

STATIC KVarArea *KVarArea_initC(KVarArea *w, KMemPool *mp, KInt s0)
{
	int i = KMemPool_index(s0);
	w = KMemPool_allocP0(mp, sizeof (KVarArea), w);
	w->s1 = KMemPool_table[i];
	w->s = s0;
	w->p = KMemPool_allocIdx(mp, i);
	memset(w->p, 0, w->s1);
	w->mp = mp;
	return w;
}

STATIC void *KVarArea_dest(KVarArea *w)
// ƁAinit܂ōėp͕s\ɂȂ.
{
	KMemPool_free(w->mp, w->s1, w->p);
	w->p = 0;
	return w;
}

STATIC void KVarArea_destFr(KVarArea *w)
{
	KMemPool_free(w->mp, sizeof (KVarArea), KVarArea_dest(w));
}

STATIC KVarArea *KVarArea_initAd(KVarArea *w, KMemPool *mp, KInt s0)
{
	w = KVarArea_init(w, mp, s0);
	KMemPool_addDest(mp, KVarArea_dest, w, 0);
	return w;
}

STATIC KVarArea *KVarArea_initAdC(KVarArea *w, KMemPool *mp, KInt s0)
{
	w = KVarArea_initC(w, mp, s0);
	KMemPool_addDest(mp, KVarArea_dest, w, 0);
	return w;
}

STATIC KVarArea *KVarArea_initAr(KMemPool *mp, KInt s0)
{
	KVarArea *w = KVarArea_init(0, mp, s0);
	KMemPool_addDest(mp, KVarArea_destFr, w, 0);
	return w;
}

STATIC KVarArea *KVarArea_initArC(KMemPool *mp, KInt s0)
{
	KVarArea *w = KVarArea_initC(0, mp, s0);
	KMemPool_addDest(mp, KVarArea_destFr, w, 0);
	return w;
}

STATIC KVarArea *KVarArea_reset(KVarArea *w, KInt s0)
{
	KMemPool_free(w->mp, w->s1, w->p);
	return KVarArea_init(w, w->mp, s0);
}

STATIC KVarArea *KVarArea_resetC(KVarArea *w, KInt s0)
{
	KMemPool_free(w->mp, w->s1, w->p);
	return KVarArea_initC(w, w->mp, s0);
}

STATIC KVarArea *KVarArea_reserve(KVarArea *w, KInt s0)
{
	if (w->s1 < s0) {
		KInt s2 = w->s1 << 1;
		while (s2 < s0)
			s2 <<= 1;
		w->p = KMemPool_realloc(w->mp, w->s1, s2, w->p);
		w->s1 = s2;
	}
	return w;
}

STATIC KVarArea *KVarArea_reserveC(KVarArea *w, KInt s0)
{
	if (w->s1 < s0) {
		KInt s2 = w->s1 << 1;
		while (s2 < s0)
			s2 <<= 1;
		w->p = KMemPool_reallocC(w->mp, w->s1, s2, w->p);
		w->s1 = s2;
	}
	return w;
}

STATIC void *KVarArea_reserve1(KVarArea *w, KInt s0)
{
	s0 += w->s;
	if (w->s1 < s0)
		KVarArea_reserve(w, s0);
	void *q = w->p + w->s;
	w->s = s0;
	return q;
}

STATIC void *KVarArea_reserve1C(KVarArea *w, KInt s0)
{
	s0 += w->s;
	if (w->s1 < s0)
		KVarArea_reserveC(w, s0);
	void *q = w->p + w->s;
	w->s = s0;
	return q;
}

STATIC KVarArea *KVarArea_expand2(KVarArea *w)
{
	w->p = KMemPool_realloc(w->mp, w->s1, w->s1 << 1, w->p);
	w->s1 <<= 1;
	return w;
}

STATIC KVarArea *KVarArea_expand2C(KVarArea *w)
{
	w->p = KMemPool_reallocC(w->mp, w->s1, w->s1 << 1, w->p);
	w->s1 <<= 1;
	return w;
}

STATIC KVarArea *KVarArea_addPtr(KVarArea *w, void *p)
{
	*((void **) KVarArea_reserve1(w, sizeof (void *))) = p;
	return w;	
}

STATIC KVarArea *KVarArea_addChar(KVarArea *w, char c)
{
	*((char *) KVarArea_reserve1(w, sizeof (char))) = c;
	return w;
}

STATIC KVarArea *KVarArea_addInt(KVarArea *w, KInt i)
{
	*((KInt *) KVarArea_reserve1(w, sizeof (KInt))) = i;
	return w;
}

STATIC KVarArea *KVarArea_addBytes(KVarArea *w, KInt s, const char *p)
{
	if (s > 0)
		memcpy(KVarArea_reserve1(w, s), p, s);
	return w;
}

STATIC KVarArea *KVarArea_insPtrIdx0(KVarArea *w, KInt j, void *p)
{
	if (w->s + (KInt) sizeof (void *) > w->s1)
		KVarArea_reserve(w, w->s + sizeof (void *));
	if (j < w->s)
		memmove(w->p + j + sizeof (void *), w->p + j, w->s - j);
	*((void **) &w->p[j]) = p;
	w->s += sizeof (void *);
	return w;	
}

STATIC KVarArea *KVarArea_delPtrIdx0(KVarArea *w, KInt j)
{
	w->s -= sizeof (void *);
	if (j < w->s)
		memmove(w->p + j, w->p + j + sizeof (void *), w->s - j);
	return w;	
}

STATIC KVarArea *KVarArea_delPtrIdx(KVarArea *w, KInt j)
{
	return KVarArea_delPtrIdx0(w, j * sizeof (void *));
}

STATIC void *KVarArea_getPtrIdx(KVarArea *w, KInt j)
{
	return ((void **) w->p)[j];
}

STATIC KVarArea *KVarArea_setPtrIdx(KVarArea *w, KInt j, void *p)
{
	((void **) w->p)[j] = p;
	return w;
}

STATIC KInt KVarArea_getIntIdx(KVarArea *w, KInt j)
{
	return ((KInt *) w->p)[j];
}

STATIC KVarArea *KVarArea_setIntIdx(KVarArea *w, KInt j, KInt i)
{
	((KInt *) w->p)[j] = i;
	return w;
}

STATIC KInt KVarArea_sizPtr(KVarArea *w)
{
	return w->s / sizeof (void *);
}

STATIC KInt KVarArea_sizInt(KVarArea *w)
{
	return w->s / sizeof (KInt);
}

//////////////////////////////////////////////////////////////////////////////// KIndexS

typedef struct KIndexS_ {
	KVarArea va;
	void *cmp, *opt;
} KIndexS;

STATIC KIndexS *KIndexS_init(KIndexS *w, KMemPool *mp, void *cmp, void *opt, int s1)
{
	w = KMemPool_allocP0(mp, sizeof (KMemPool), w);
	KVarArea_init(&w->va, mp, s1 * sizeof (void *));
	w->cmp = cmp;
	w->opt = opt;
	return w;
}

STATIC void *KIndexS_dest(KIndexS *w)
{
	KVarArea_dest(&w->va);
	return w;
}

STATIC void KIndexS_destFr(KIndexS *w)
{
	KMemPool_free(w->va.mp, sizeof (KIndexS), KIndexS_dest(w));
}

STATIC KIndexS *KIndexS_initAd(KIndexS *w, KMemPool *mp, void *cmp, void *opt, int s1)
{
	w = KIndexS_init(w, mp, cmp, opt, s1);
	KMemPool_addDest(mp,  KIndexS_dest, w, 0);
	return w;
}

STATIC KIndexS *KIndexS_initAr(KMemPool *mp, void *cmp, void *opt, int s1)
{
	KIndexS *w = KIndexS_init(0, mp, cmp, opt, s1);
	KMemPool_addDest(mp,  KIndexS_destFr, w, 0);
	return w;
}

STATIC KInt KIndexS_search0(KVarArea *va, void *cmp_, void *opt, void *k, KInt *cr)
// vfԍł͂ȂA8/4{Ԃ(index0).
{
	KInt a, b, c, cc, cc1;
	KInt (*cmp)(void *, void *, void *) = cmp_;
	if (va->s <= 0) {
		*cr = -1;
		return 0;
	}
	cc = cmp(k, *((void **) va->p), opt);
	if (cc <= 0) {
		*cr = cc;
		return 0;
	}
	cc = cmp(k, *((void **) (va->p + va->s - sizeof (void *))), opt);
	if (cc > 0) {
		*cr = -1;
		return va->s;
	}
	if (cc == 0) {
		*cr = 0;
		return va->s - sizeof (void *);
	}
	a = 0;
	b = va->s - sizeof (void *);
	for (;;) {
		c = ((b - a) >> 1) & (- sizeof (void *));
		if (c == 0) break;
		c += a;
		cc1 = cmp(k, *((void **) (va->p + c)), opt);
		if (cc1 > 0)
			a = c;
		else {
			b = c;
			cc = cc1;
		}
	}
	*cr = cc;
	return b;
}

STATIC KInt KIndexS_search(KIndexS *w, void *k, KInt *cr)
{
	return KIndexS_search0(&w->va, w->cmp, w->opt, k, cr);
}

STATIC KInt KIndexS_ins(KIndexS *w, void *d)
{
	KInt cr, i = KIndexS_search0(&w->va, w->cmp, w->opt, d, &cr);
//	if (cr == 0) return ~i;
	KVarArea_insPtrIdx0(&w->va, i, d);
	return i;
}

STATIC KInt KIndexS_del(KIndexS *w, void *k)
{
	KInt cr, i = KIndexS_search0(&w->va, w->cmp, w->opt, k, &cr);
	if (cr != 0) return -1;
	KVarArea_delPtrIdx0(&w->va, i);
	return i;
}

STATIC KIndexS *KIndexS_add(KIndexS *w, void *d)
{
	KVarArea_addPtr(&w->va, d);
	return w;
}

STATIC void *KIndexS_optStatic;
STATIC KInt (*KIndexS_cmpStatic)(const void *, const void *, void *);

STATIC int KIndexS_cmpQsort(const void **a, const void **b)
{
	#if (defined(KUSE64))
		KInt i = KIndexS_cmpStatic(*a, *b, KIndexS_optStatic);
		return i >> 63 | ((i > 0) & 1);
	#elif (defined(KUSE32))
		return KIndexS_cmpStatic(*a, *b, KIndexS_optStatic);
	#endif
}

STATIC KIndexS *KIndexS_sort(KIndexS *w)
{
	KIndexS_cmpStatic = (KInt (*)(const void *, const void *, void *)) w->cmp;
	KIndexS_optStatic = w->opt;
	qsort(w->va.p, w->va.s / sizeof (void *), sizeof (void *), (int (*)(const void *, const void *)) KIndexS_cmpQsort);
	return w;
}

//////////////////////////////////////////////////////////////////////////////// KIndexHS

typedef struct KIndexHSNode_ {
	KUInt h;
	void *d;
} KIndexHSNode;

typedef struct KIndexHS_ {
	KVarArea **p;
	KInt n, n1, r, n1r;
	void *hsh, *hshOpt;
	void *cmp, *cmpOpt;
	KMemPool *mp;
	int idxVA;
} KIndexHS;

STATIC KInt KIndexHS_cmpEx(KIndexHSNode *a, KIndexHSNode *b, KIndexHS *w)
{
	if (a->h != b->h) {
		if (a->h > b->h) return +1; else return -1;
	}
	KInt (*cmp)(void *, void *, void *) = (KInt (*)(void *, void *, void *)) w->cmp;
	return cmp(a->d, b->d, w->cmpOpt);
}

STATIC void *KIndexHS_optStatic;
STATIC KInt (*KIndexHS_cmpStatic)(const void *, const void *, void *);

int KIndexHS_cmpQsort(KIndexHSNode *a, KIndexHSNode *b)
{
	if (a->h != b->h) {
		if (a->h > b->h) return +1; else return -1;
	}
	#if (defined(KUSE64))
		KInt i = KIndexHS_cmpStatic(a->d, b->d, KIndexHS_optStatic);
		return i >> 63 | ((i > 0) & 1);
	#elif (defined(KUSE32))
		return KIndexHS_cmpStatic(a->d, b->d, KIndexHS_optStatic);
	#endif
}

STATIC KInt KIndexHS_search00(KVarArea *va, KInt (*cmp)(void *, void *, void *), void *opt, void *k, KInt *cr)
// vfԍł͂ȂA8/4{Ԃ(index0).
{
	KInt a, b, c, cc, cc1;
#if 0
	if (va->s <= 0) {
		*cr = -1;
		return 0;
	}
#endif
	cc = cmp(k, va->p, opt);
	if (cc <= 0) {
		*cr = cc;
		return 0;
	}
	if (va->s <= 4 * (int) sizeof (KIndexHSNode)) {
		for (a = sizeof (KIndexHSNode); a < va->s; a += sizeof (KIndexHSNode)) {
			cc = cmp(k, va->p + a, opt);
			if (cc <= 0) {
				*cr = cc;
				return a;
			}
		}
		*cr = -1;
		return va->s;
	}
	cc = cmp(k, va->p + va->s - sizeof (KIndexHSNode), opt);
	if (cc > 0) {
		*cr = -1;
		return va->s;
	}
	if (cc == 0) {
		*cr = 0;
		return va->s - sizeof (KIndexHSNode);
	}
	a = 0;
	b = va->s - sizeof (KIndexHSNode);
	for (;;) {
		c = ((b - a) >> 1) & (- sizeof (KIndexHSNode));
		if (c == 0) break;
		c += a;
		cc1 = cmp(k, va->p + c, opt);
		if (cc1 > 0)
			a = c;
		else {
			b = c;
			cc = cc1;
		}
	}
	*cr = cc;
	return b;
}

STATIC KVarArea *KIndexHS_add0(KIndexHS *w, KIndexHSNode n)
{
	KVarArea **p0 = w->p + (n.h & w->n1), *va = *p0;
	if (va == 0) {
		*p0 = va = KMemPool_allocIdx(w->mp, w->idxVA);
		KVarArea_init(va, w->mp, 0);
	}
	KVarArea_reserve(va, va->s + (sizeof (KIndexHSNode)));
	*((KIndexHSNode *) (va->p + va->s)) = n;
	va->s += (sizeof (KIndexHSNode));
	return va;
}

STATIC KIndexHS *KIndexHS_rehash(KIndexHS *w, KInt n1);

STATIC KIndexHS *KIndexHS_init(KIndexHS *w, KMemPool *mp, void *hsh, void *hshOpt, void *cmp, void *cmpOpt, KInt r, KInt n1)
// 悭킩Ȃ΁Arn10ɂĂĂ悢. ŃftHglݒ肷.
{
	w = KMemPool_allocP0(mp, sizeof (KIndexHS), w);
	w->p = 0;
	w->n = 0;
	if (r < 5)
		r = 5;
	w->r = r;
	w->hsh = hsh;
	w->hshOpt = hshOpt;
	w->cmp = cmp;
	w->cmpOpt = cmpOpt;
	w->mp = mp;
	w->idxVA = KMemPool_index(sizeof (KVarArea));
	KIndexHS_rehash(w, n1);
	return w;
}

STATIC void *KIndexHS_dest(KIndexHS *w)
{
	KVarArea **p0 = w->p;
	KInt i;
	for (i = 0; i <= w->n1; i++) {
		if (p0[i] != 0)
			KMemPool_freeIdx(w->mp, w->idxVA, KVarArea_dest(p0[i]));
	}
	KMemPool_free(w->mp, (w->n1 + 1) * sizeof (KVarArea *), p0);
	return w;
}

STATIC void KIndexHS_destFr(KIndexHS *w)
{
	KMemPool_free(w->mp, sizeof (KIndexHS), KIndexHS_dest(w));
}

STATIC KIndexHS *KIndexHS_initAd(KIndexHS *w, KMemPool *mp, void *hsh, void *hshOpt, void *cmp, void *cmpOpt, KInt r, KInt n1)
{
	w = KIndexHS_initAd(w, mp, hsh, hshOpt, cmp, cmpOpt, r, n1);
	KMemPool_addDest(mp, KIndexHS_dest, w, 0);
	return w;
}

STATIC KIndexHS *KIndexHS_initAr(KIndexHS *w, KMemPool *mp, void *hsh, void *hshOpt, void *cmp, void *cmpOpt, KInt r, KInt n1)
{
	w = KIndexHS_initAd(w, mp, hsh, hshOpt, cmp, cmpOpt, r, n1);
	KMemPool_addDest(mp, KIndexHS_destFr, w, 0);
	return w;
}

STATIC KIndexHS *KIndexHS_reset(KIndexHS *w)
{
	KVarArea **p0 = w->p;
	KInt i;
	for (i = 0; i <= w->n1; i++) {
		if (p0[i] != 0)
			KMemPool_freeIdx(w->mp, w->idxVA, KVarArea_dest(p0[i]));
	}
	memset(w->p, 0, (w->n1 + 1) * sizeof (KVarArea *));
	w->n = 0;
	return w;
}

STATIC KIndexHS *KIndexHS_sort(KIndexHS *w);

STATIC KIndexHS *KIndexHS_rehash(KIndexHS *w, KInt n1)
{
	KInt n10 = w->n1 + 1, i, j, j1;
	KVarArea **p0 = w->p;
	KIndexHSNode *p2;
	if (n1 < 16)
		n1 = 16;
	if ((n1 & (n1 - 1)) != 0) {
		#if (defined(KUSE64))
			n1 = kgetMsb64(n1) << 1;
		#elif (defined(KUSE32))
			n1 = kgetMsb32(n1) << 1;
		#endif
	}
	w->p = KMemPool_allocC(w->mp, n1 * sizeof (KVarArea *));
	#if (defined(KUSE64))
		w->n1r = 0x7fffffffffffffff;
	#elif (defined(KUSE32))
		w->n1r = 0x7fffffff;
	#endif
	if (w->r > 0)
		w->n1r = 4 * n1 / w->r;
	w->n1 = --n1;
	if (p0 != 0) {
		for (i = 0; i < n10; i++) {
			if (p0[i] != 0) {
				j1 = p0[i]->s / sizeof (KIndexHSNode);
				p2 = (KIndexHSNode *) p0[i]->p;
				for (j = 0; j < j1; j++)
					KIndexHS_add0(w, p2[j]);
				KMemPool_freeIdx(w->mp, w->idxVA, KVarArea_dest(p0[i]));
			}
		}
		KMemPool_free(w->mp, n10 * sizeof (KVarArea *), p0);
		KIndexHS_sort(w);
	}
	return w;
}

STATIC void *KIndexHS_search0(KIndexHS *w, void *k, KUInt h)
{
	KInt i, cr;
	KIndexHSNode n;
	KVarArea **p0, *va;
	n.h = h;
	n.d = k;
	p0 = w->p + (n.h & w->n1);
	va = *p0;
	if (va == 0)
		return 0;
	i = KIndexHS_search00(va, (KInt (*)(void *, void *, void *)) KIndexHS_cmpEx, w, &n, &cr);
	if (cr == 0)
		return ((KIndexHSNode *) (va->p + i))->d;
	return 0;
}

STATIC void *KIndexHS_search1(KIndexHS *w, void *k)
{
	KUInt (*hsh)(void *, void *) = (KUInt (*)(void *, void *)) w->hsh;
	return KIndexHS_search0(w, k, hsh(k, w->hshOpt));
}

STATIC KVarArea *KIndexHS_add(KIndexHS *w, void *d)
{
	KUInt (*hsh)(void *, void *) = (KUInt (*)(void *, void *)) w->hsh;
	KIndexHSNode q;
	q.h = hsh(d, w->hshOpt);
	q.d = d;
	if (w->n >= w->n1r)
		KIndexHS_rehash(w, (w->n1 + 1) * 2);
	w->n++;
	return KIndexHS_add0(w, q);
}

STATIC KIndexHS *KIndexHS_sort(KIndexHS *w)
{
	KIndexHS_cmpStatic = (KInt (*)(const void *, const void *, void *)) w->cmp;
	KIndexHS_optStatic = w->cmpOpt;
	KInt i;
	for (i = 0; i <= w->n1; i++) {
		if (w->p[i] != 0)
			qsort(w->p[i]->p, w->p[i]->s / sizeof (KIndexHSNode), sizeof (KIndexHSNode), (int (*)(const void *, const void *)) KIndexHS_cmpQsort);
	}
	return w;
}

STATIC KIndexHS *KIndexHS_ins(KIndexHS *w, void *d)
{
	KVarArea *va = KIndexHS_add(w, d);
	KIndexHS_cmpStatic = (KInt (*)(const void *, const void *, void *)) w->cmp;
	KIndexHS_optStatic = w->cmpOpt;
	qsort(va->p, va->s / sizeof (KIndexHSNode), sizeof (KIndexHSNode), (int (*)(const void *, const void *)) KIndexHS_cmpQsort);
	return w;
}

STATIC void *KIndexHS_del(KIndexHS *w, void *k)
{
	KUInt (*hsh)(void *, void *) = (KUInt (*)(void *, void *)) w->hsh;
	KIndexHSNode q;
	KVarArea **p0, *va;
	KInt i, cr;
	q.h = hsh(k, w->hshOpt);
	q.d = k;
	p0 = w->p + (q.h & w->n1);
	va = *p0;
	if (va == 0) return 0;
	i = KIndexHS_search00(va, (KInt (*)(void *, void *, void *)) KIndexHS_cmpEx, w, &q, &cr);
	if (cr != 0) return 0;
	k = ((KIndexHSNode *) (va->p + i))->d;
	if (va->s > (int) sizeof (KIndexHSNode))
		KVarArea_delPtrIdx0(va, i);
	else {
		KMemPool_freeIdx(w->mp, w->idxVA, KVarArea_dest(va));
		*p0 = 0;
	}
	return k;
}

//////////////////////////////////////////////////////////////////////////////// KIndexDict0

typedef KIndexHS KIndexDict0;	// ofs=0, IuWFNg, Œ蒷.

#define KIndexDict0_hsh 	kfnvU
#define KIndexDict0_cmp		memcmp

STATIC KIndexDict0 *KIndexDict0_init(KIndexDict0 *w, KMemPool *mp, KInt l, KInt r, KInt n1)
// 悭킩Ȃ΁Arn10ɂĂĂ悢. ŃftHglݒ肷.
{
	return KIndexHS_init(w, mp, KIndexDict0_hsh, (void *) l, KIndexDict0_cmp, (void *) l, r, n1);
}

#define KIndexDict0_dest	KIndexHS_dest
#define KIndexDict0_destFr	KIndexHS_destFr

STATIC KIndexDict0 *KIndexDict0_initAd(KIndexDict0 *w, KMemPool *mp, KInt l, KInt r, KInt n1)
{
	w = KIndexDict0_init(w, mp, l, r, n1);
	KMemPool_addDest(mp, KIndexDict0_dest, w, 0);
	return w;
}

STATIC KIndexDict0 *KIndexDict0_initAr(KMemPool *mp, KInt l, KInt r, KInt n1)
{
	KIndexDict0 *w = KIndexDict0_init(0, mp, l, r, n1);
	KMemPool_addDest(mp, KIndexDict0_destFr, w, 0);
	return w;
}

#define KIndexDict0_reset		KIndexHS_reset
#define KIndexDict0_rehash		KIndexHS_rehash
#define KIndexDict0_search0		KIndexHS_search0
#define KIndexDict0_search1		KIndexHS_search1
#define KIndexDict0_add			KIndexHS_add
#define KIndexDict0_sort		KIndexHS_sort
#define KIndexDict0_ins			KIndexHS_ins
#define KIndexDict0_del			KIndexHS_del

//////////////////////////////////////////////////////////////////////////////// KIndexDict3

typedef KIndexHS KIndexDict3;	// ofs=0, IuWFNgO, ϒ. (s, p)IuWFNgɂ.

STATIC KUInt KIndexDict3_hsh(KInt *k)
{
	return kfnvU((char *) k[1], k[0]);
}

STATIC KInt KIndexDict3_cmp(KInt *a, KInt *b)
{
	KInt la = *a, lb = *b;
	if (la < lb) return -1;
	if (la > lb) return +1;
	return memcmp((char *) a[1], (char *) b[1], la);
}

STATIC KIndexDict3 *KIndexDict3_init(KIndexDict3 *w, KMemPool *mp, KInt r, KInt n1)
// 悭킩Ȃ΁Arn10ɂĂĂ悢. ŃftHglݒ肷.
{
	return KIndexHS_init(w, mp, KIndexDict3_hsh, 0, KIndexDict3_cmp, 0, r, n1);
}

#define KIndexDict3_dest	KIndexHS_dest
#define KIndexDict3_destFr	KIndexHS_destFr

STATIC KIndexDict3 *KIndexDict3_initAd(KIndexDict3 *w, KMemPool *mp, KInt r, KInt n1)
{
	w = KIndexDict3_init(w, mp, r, n1);
	KMemPool_addDest(mp, KIndexDict3_dest, w, 0);
	return w;
}

STATIC KIndexDict3 *KIndexDict3_initAr(KMemPool *mp, KInt r, KInt n1)
{
	KIndexDict3 *w = KIndexDict3_init(0, mp, r, n1);
	KMemPool_addDest(mp, KIndexDict3_destFr, w, 0);
	return w;
}

#define KIndexDict3_reset		KIndexHS_reset
#define KIndexDict3_rehash		KIndexHS_rehash
#define KIndexDict3_search0		KIndexHS_search0
#define KIndexDict3_search1		KIndexHS_search1
#define KIndexDict3_add			KIndexHS_add
#define KIndexDict3_sort		KIndexHS_sort
#define KIndexDict3_ins			KIndexHS_ins
#define KIndexDict3_del			KIndexHS_del

STATIC void KIndexDict3_dump(KIndexDict3 *w)
{
	printf("n1=%d\n", (int) w->n1);
	KInt i;
	for (i = 0; i <= w->n1; i++) {
		if (w->p[i] != 0) {
			KIndexHSNode *n = (KIndexHSNode *) w->p[i]->p;
			KInt j, j1 = w->p[i]->s / sizeof (KIndexHSNode);
			for (j = 0; j < j1; j++) {
				printf("#%04x: h=" KPRINTF_FORMAT("%08x", "%016I64x", "%016lx") " d=%08x ", (int) i, n[j].h, (int) (KInt) n[j].d);
				KInt *e = n[j].d;
				printf("kfnvU=" KPRINTF_FORMAT("%08x", "%016I64x", "%016lx") "\n", kfnvU((void *) e[1], e[0]));
			}
		}
	}
}
